import{n as t,M as d,j as c,b5 as l,b6 as g,J as e}from"./index.B7J6suR-.js";const f=t("iframe",{target:"e1q9bkxk0"})(({theme:o,disableScrolling:n})=>({colorScheme:"normal",border:"none",padding:o.spacing.none,margin:o.spacing.none,overflow:n?"hidden":void 0}));function F({element:o,width:n}){const i=o.hasWidth?o.width:n,r=s(o.src),a=d(r)?void 0:s(o.srcdoc);return c(f,{className:"stIFrame","data-testid":"stIFrame",allow:l,disableScrolling:!o.scrolling,src:r,srcDoc:a,width:i,height:o.height,scrolling:o.scrolling?"auto":"no",sandbox:g,title:"st.iframe"})}function s(o){return e(o)||o===""?void 0:o}export{F as default};
