import{b9 as a,j as e,ba as c,br as B,b8 as b,bb as f}from"./index.B7J6suR-.js";function p(s){const{disabled:l,element:t,widgetMgr:o,width:n,fragmentId:r}=s,d={width:n};let i=a.SECONDARY;t.type==="primary"?i=a.PRIMARY:t.type==="tertiary"&&(i=a.TERTIARY);const u=t.help?n:!0;return e("div",{className:"stButton","data-testid":"stButton",style:d,children:e(c,{help:t.help,children:e(B,{kind:i,size:b.SMALL,disabled:l,fluidWidth:t.useContainerWidth?u:!1,onClick:()=>o.setTriggerValue(t,{fromUi:!0},r),children:e(f,{icon:t.icon,label:t.label})})})})}export{p as default};
