import{n as t,be as e,z as o,j as a,aE as i}from"./index.B7J6suR-.js";import{P as l}from"./ProgressBar.Di2oT-20.js";const n=t("div",{target:"ejagcfm0"})(({theme:s})=>({paddingBottom:s.spacing.sm,lineHeight:"normal",color:e(s)}));function c({element:s,width:r}){return o("div",{className:"stProgress","data-testid":"stProgress",children:[a(n,{children:a(i,{source:s.text,allowHTML:!1,isLabel:!0})}),a(l,{value:s.value,width:r})]})}export{c as default};
