import{c9 as c,ca as u,aq as s,aV as f}from"./index.B7J6suR-.js";function n(r,t){(t==null||t>r.length)&&(t=r.length);for(var e=0,o=Array(t);e<t;e++)o[e]=r[e];return o}function l(r,t){if(r){if(typeof r=="string")return n(r,t);var e={}.toString.call(r).slice(8,-1);return e==="Object"&&r.constructor&&(e=r.constructor.name),e==="Map"||e==="Set"?Array.from(r):e==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(e)?n(r,t):void 0}}function y(r,t){if(!(r instanceof t))throw new TypeError("Cannot call a class as a function")}function a(r,t){for(var e=0;e<t.length;e++){var o=t[e];o.enumerable=o.enumerable||!1,o.configurable=!0,"value"in o&&(o.writable=!0),Object.defineProperty(r,c(o.key),o)}}function b(r,t,e){return t&&a(r.prototype,t),e&&a(r,e),Object.defineProperty(r,"prototype",{writable:!1}),r}function _(r,t){if(typeof t!="function"&&t!==null)throw new TypeError("Super expression must either be null or a function");r.prototype=Object.create(t&&t.prototype,{constructor:{value:r,writable:!0,configurable:!0}}),Object.defineProperty(r,"prototype",{writable:!1}),t&&u(r,t)}function i(r){return i=Object.setPrototypeOf?Object.getPrototypeOf.bind():function(t){return t.__proto__||Object.getPrototypeOf(t)},i(r)}function d(r,t){if(t&&(s(t)=="object"||typeof t=="function"))return t;if(t!==void 0)throw new TypeError("Derived constructors may only return object or undefined");return f(r)}export{_,y as a,b,i as c,d,l as e,n as f};
